# -*- coding: utf-8 -*-

"""Main module."""

import paramiko
import keyring
from sshtunnel import SSHTunnelForwarder
from issho.helpers import absolute_path, default_sftp_path
from issho.config import read_issho_conf
import sys
import time
from shutil import copyfile


class Issho:

    def __init__(self,
                 key_path='~/.ssh/id_rsa',
                 ssh_config_path="~/.ssh/config",
                 profile='dev',
                 kinit=True):
        self.key_path = key_path
        self.ssh_config_path = ssh_config_path
        self.profile = profile
        self.remote_conf = read_issho_conf(profile)
        self.ssh_conf = self._get_issho_ssh_config()
        self.hostname = self.ssh_conf['hostname']
        self.port = int(self.ssh_conf['port'])
        self.user = self.ssh_conf['user']
        self._ssh = self._connect()
        if kinit:
            self.kinit()
        self._remote_home_dir = self.get_output('echo $HOME').strip()
        return

    def _get_pkey(self):
        """
        Helper for getting an RSA key
        """
        key_file = absolute_path(self.key_path)
        return paramiko.RSAKey.from_private_key_file(
            key_file, password=keyring.get_password('SSH', key_file))

    def _get_issho_ssh_config(self):
        """
        Helper method for getting data from .ssh/config
        """
        ssh_config_file = absolute_path(self.ssh_config_path)
        conf = paramiko.SSHConfig()
        conf.parse(open(ssh_config_file))
        issho_conf = conf.lookup(self.profile)
        return issho_conf

    def local_forward(self, remote_host, remote_port, local_host='0.0.0.0', local_port=44556):
        """
        Forwards a port from a remote through this Issho object.
        Useful for connecting to remote hosts that can only be accessed
        from inside a VPC of which your devbox is part.
        """
        tunnel = SSHTunnelForwarder(
            (self.hostname, self.port),
            ssh_username=self.user,
            ssh_pkey=self._get_pkey(),
            remote_bind_address=(remote_host, remote_port),
            local_bind_address=(local_host, local_port))
        tunnel.start()
        return tunnel

    def _connect(self):
        """
        Uses paramiko to connect to the remote specified
        :return:
        """
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(self.hostname,
                    username=self.user,
                    port=self.port,
                    pkey=self._get_pkey())
        return ssh

    def exec(self, cmd, bg=False, debug=False, capture_output=False):
        """
        Executes a command in bash over the SSH connection

        Note, this command does not use an interactive terminal;
        it instead uses a *non-interactive login* shell.
        This means (specifically) that your aliased commands will not work
        and only variables exported in your remote .bashrc will be available.

        :param cmd: The bash command to be run remotely

        :param bg: True = run in the background

        :param debug: True = print some debugging output

        :param capture_output: True = return stdout as a string

        :return:
        """
        if bg:
            cmd = 'cmd=$"{}"; nohup bash -c "$cmd" &'.format(cmd.replace('"', r'\"'))
        if debug:
            print(cmd)
        stdin, stdout, stderr = self._ssh.exec_command(cmd)

        captured_output = ''
        for line in stdout:
            if capture_output:
                captured_output += line
            else:
                print(line, end='')

        for line in stderr:
            sys.stderr.write(line)
        return captured_output

    def exec_bg(self, cmd, **kwargs):
        """
        Syntactic sugar for exec(bg=True)
        """
        return self.exec(cmd, bg=True, **kwargs)

    def get_output(self, cmd, **kwargs):
        """
        Syntactic sugar for exec(capture_output=True)
        """
        return self.exec(cmd, capture_output=True, **kwargs)

    def get(self, remotepath, localpath=None):
        """
        Gets the file at the remote path and puts it locally.

        :param remotepath: The path on the remote from which to get.

        :param localpath: Defaults to the name of the remote path
        """
        paths = self._sftp_paths(localpath=localpath, remotepath=remotepath)
        with self._ssh.open_sftp() as sftp:
            sftp.get(remotepath=paths['remotepath'], localpath=paths['localpath'])
        return

    def put(self, localpath, remotepath=None):
        """
        Puts the file at the local path to the remote.

        :param localpath: The local path of the file to put to the remote

        :param remotepath: Defaults to the name of the local path
        """
        paths = self._sftp_paths(localpath=localpath, remotepath=remotepath)
        with self._ssh.open_sftp() as sftp:
            sftp.put(localpath=paths['localpath'], remotepath=paths['remotepath'], callback=self._sftp_progress)
        return

    def kinit(self):
        """
        Runs kerberos init
        """
        kinit_pw = keyring.get_password('{}_kinit'.format(self.profile), self.user)
        if kinit_pw:
            self.exec('echo {} | kinit'.format(kinit_pw))
        else:
            raise OSError("Add your kinit password to your keyring")
        return

    def hive(self, query):
        """
        Runs a hive query using the parameters
        set in .issho/config.toml

        :param query: a string query, or the name of a query file
        name to run.
        """
        tmp_filename = '/tmp/issho_{}.sql'.format(time.time())
        if query.endswith('sql', 'hql'):
            copyfile(query, tmp_filename)
        else:
            with open(tmp_filename, 'w') as f:
                f.write(query)
        self.put(tmp_filename, tmp_filename)

        self.exec('beeline {opts} -u  "{jdbc}" -f {fn}'.format(
            opts=self.remote_conf['HIVE_OPTS'],
            jdbc=self.remote_conf['HIVE_JDBC'],
            fn=tmp_filename))

    def _sftp_paths(self, localpath, remotepath):
        localpath = default_sftp_path(localpath, remotepath)
        remotepath = default_sftp_path(remotepath, localpath)
        return {'localpath': str(localpath.expanduser()),
                'remotepath': str(remotepath).replace('~', self._remote_home_dir)}

    @staticmethod
    def _sftp_progress(transferred, remaining):
        print('{:.1f} MB transferred, {:.1f} MB remaining'.format(transferred/2**20, remaining/2**20))




from prompt_toolkit import prompt
import keyring
from issho.config import write_issho_conf
import os
import fire


class IsshoCLI:
    """
    CLI for Issho; right now only used for configuration
    """

    def config(self, profile):
        """
        Configures a single issho profile.

        :param profile: name of the profile to configure; should
        match the name in the ssh config.
        """

        while True:
            pw = prompt("Enter the profile's kinit password: ", is_password=True)
            if not pw:
                break
            pw2 = prompt('Enter the kinit password again: ', is_password=True)
            if pw != pw2:
                print('passwords do not match')
            else:
                keyring.set_password('{}_kinit'.format(profile), os.environ.get('USER'), pw)
                break

        hive_opts = prompt('Hive Options: ')
        hive_jdbc = prompt('Hive JDBC connection string: ')
        spark_shell_conf = prompt('Spark Shell Configuration String: ')

        new_conf = {
          'HIVE_OPTS': hive_opts,
          'HIVE_JDBC': hive_jdbc,
          'SPARK_CONF': spark_shell_conf
        }
        write_issho_conf({profile: new_conf})


def main():
    """
    Inititates the CLI using python-fire
    """
    fire.Fire(IsshoCLI)


if __name__ == "__main__":
    main()



from pathlib import Path
import socket


def absolute_path(raw_path):
    """
    Gets the string absolute path from
    :param raw_path: a string or pathlib.Path object
    """
    if not isinstance(raw_path, Path):
        raw_path = Path(raw_path)
    return str(raw_path.expanduser())


def default_sftp_path(this_path, default_path):
    """
    If ``this_path`` exists, return it as a path, else, return
    the pathlib.Path.name of the default path
    """
    return Path(this_path) if this_path else Path(Path(default_path).name)


def able_to_connect(host, port, timeout=1.5):
    """
    Returns true if it is possible to connect to the specified host
    and port, within the given timeout in seconds.
    """
    try:
        sock = socket.socket()
        sock.settimeout(timeout)
        sock.connect((host, port))
    except Exception as e:
        return False
    return True



# -*- coding: utf-8 -*-

"""Top-level package for issho."""

__author__ = """Michael Bilow"""
__email__ = 'michael.k.bilow@gmail.com'
__version__ = '0.2.1'
__docformat__ = 'restructuredtext'


from issho.issho import Issho

# module level doc-string
__doc__ = """
issho - simple connections to remote machines
=====================================================================
**issho** is a Python package providing a simple wrapper over
paramiko, providing = operators interacting with remote machines 

Main Features
-------------
Here are a few of the things that issho (should) do well:
  - execute commands on a remote box
  - run commands in the background on a remote box easily
  - transfer files to and from a remote easily
  - set up an SSH tunnel through a remote

TODOs
---------
  - make it easy to interact with hadoop
  - make it easy to configure new services
  - make it easy to add plugins to issho
"""



import toml
from pathlib import Path

ISSHO_CONF_FILE = Path.home().joinpath('.issho').joinpath('conf.toml')


def _make_issho_conf_dir():
    if not ISSHO_CONF_FILE.exists():
        ISSHO_CONF_FILE.parent.mkdir()
        ISSHO_CONF_FILE.touch()
    return


def read_issho_conf(host):
    _make_issho_conf_dir()
    conf = toml.load(ISSHO_CONF_FILE)
    if host not in conf:
        raise ValueError
    return conf[host]


def write_issho_conf(new_conf_dict):
    _make_issho_conf_dir()
    old_issho_conf = toml.load(ISSHO_CONF_FILE)
    new_conf = {**old_issho_conf, **new_conf_dict}
    toml.dump(new_conf, open(str(ISSHO_CONF_FILE), 'w'))
    return



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'sshtunnel>=0.1.4',
    'keyring>=18.0.0',
    'paramiko>=2.4.0',
    'cryptography==2.4.2',  # https://github.com/paramiko/paramiko/issues/1369
    'prompt_toolkit>=2.0.5',
    'toml>=0.10.0',
    'fire>=0.1.3'
]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="Michael Bilow",
    author_email='michael.k.bilow@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="issho allows easy access to simple commands on a remote machine.",
    entry_points='''
        [console_scripts]
        issho=issho.cli:main
    ''',
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='issho,paramiko,keyring,ssh,sftp,python,python-fire,prompt-toolkit,pysftp,sshed,sshtunnel',
    name='issho',
    packages=find_packages(include=['issho']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/michaelbilow/issho',
    version='0.2.1',
    zip_safe=False,
)
